/*
    SDL - Simple DirectMedia Layer
    Copyright (C) 1997-2009 Sam Lantinga

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Sam Lantinga
    slouken@libsdl.org
*/
module org.libsdl.cdrom;

import org.libsdl.stdinc;

immutable int SDL_MAX_TRACKS = 99;

immutable Uint8 SDL_AUDIO_TRACK = 0x00;
immutable Uint8 SDL_DATA_TRACK  = 0x04;

enum CDstatus
{
	CD_TRAYEMPTY,
	CD_STOPPED,
	CD_PLAYING,
	CD_PAUSED,
	CD_ERROR = -1
}

alias CDstatus.CD_TRAYEMPTY CD_TRAYEMPTY;
alias CDstatus.CD_STOPPED CD_STOPPED;
alias CDstatus.CD_PLAYING CD_PLAYING;
alias CDstatus.CD_PAUSED CD_PAUSED;
alias CDstatus.CD_ERROR CD_ERROR;

bool CD_INDRIVE(CDstatus status)
{
	return status > 0;
}


align(4)
struct SDL_CDtrack
{
	Uint8 id;
	Uint8 type;
	Uint16 unused;
	Uint32 length;
	Uint32 offset;
}

align(4)
struct SDL_CD
{
	int id;
	CDstatus status;
	int numtracks;
	int cur_track;
	int cur_frame;
	SDL_CDtrack[SDL_MAX_TRACKS+1] track;
}

immutable int CD_FPS = 75;

void FRAMES_TO_MSF(int f, ref int M, ref int S, ref int F)
{
	F = f % CD_FPS;
	f /= CD_FPS;
	S = f % 60;
	f /= 60;
	M = f;
}

int MSF_TO_FRAMES(int M, int S, int F)
{
	return M * 60 * CD_FPS + S * CD_FPS + F;
}

extern (C)
{
	int SDL_CDNumDrives();
	const(char)* SDL_CDName(int drive);
	SDL_CD* SDL_CDOpen(int drive);
	CDstatus SDL_CDStatus(SDL_CD* cdrom);
	int SDL_CDPlayTracks(SDL_CD* cdrom,int start_track, int start_frame, int ntracks, int nframes);
	int SDL_CDPlay(SDL_CD* cdrom, int start, int length);
	int SDL_CDPause(SDL_CD* cdrom);
	int SDL_CDResume(SDL_CD* cdrom);
	int SDL_CDStop(SDL_CD* cdrom);
	int SDL_CDEject(SDL_CD* cdrom);
	void SDL_CDClose(SDL_CD* cdrom);
}
