/*
    SDL - Simple DirectMedia Layer
    Copyright (C) 1997-2009 Sam Lantinga

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Sam Lantinga
    slouken@libsdl.org
*/
module org.libsdl.rwops;

import core.stdc.stdio;
import org.libsdl.stdinc;

extern (C)
{
	align(4)
	struct SDL_RWops
	{
		int function(SDL_RWops* context, int offset, int whence) seek;
		int function(SDL_RWops* context, void* ptr, int size, int maxnum) read;
		int function(SDL_RWops* context, const(void)* ptr, int size, int num) write;
		int function(SDL_RWops* context) close;

		Uint32 type;
		union hidden
		{
			struct win32io
			{
				int append;
				void *h;
				struct buffer
				{
					void *data;
					int size;
					int left;
				}
			}

			struct stdio
			{
				int autoclose;
				FILE* fp;
			}

			struct mem
			{
				Uint8* base;
				Uint8* here;
				Uint8* stop;
			}

			struct unknown
			{
				void* data1;
			}
		}
	}
}

extern (C)
{
	SDL_RWops* SDL_RWFromFile(const(char)* file, const(char)* mode);
	SDL_RWops* SDL_RWFromFP(FILE* fp, int autoclose);
	SDL_RWops* SDL_RWFromMem(void* mem, int size);
	SDL_RWops* SDL_RWFromConstMem(const(void)* mem, int size);
	SDL_RWops* SDL_AllocRW();
	void SDL_FreeRW(SDL_RWops* area);
}

immutable int RW_SEEK_SET = 0;
immutable int RW_SEEK_CUR = 1;
immutable int RW_SEEK_END = 2;

int SDL_RWseek(SDL_RWops* ctx, int offset, int whence)
{
	return ctx.seek(ctx, offset, whence);
}

int SDL_RWtell(SDL_RWops* ctx)
{
	return ctx.seek(ctx, 0, RW_SEEK_CUR);
}

int SDL_RWread(SDL_RWops* ctx, void* ptr, int size, int n)
{
	return ctx.read(ctx, ptr, size, n);
}

int SDL_RWwrite(SDL_RWops* ctx, const(void)* ptr, int size, int n)
{
	return ctx.write(ctx, ptr, size, n);
}

int SDL_RWclose(SDL_RWops* ctx)
{
	return ctx.close(ctx);
}

extern (C)
{
	Uint16 SDL_ReadLE16(SDL_RWops* src);
	Uint16 SDL_ReadBE16(SDL_RWops* src);
	Uint32 SDL_ReadLE32(SDL_RWops* src);
	Uint32 SDL_ReadBE32(SDL_RWops* src);
	Uint64 SDL_ReadLE64(SDL_RWops* src);
	Uint64 SDL_ReadBE64(SDL_RWops* src);

	int SDL_WriteLE16(SDL_RWops* dst, Uint16 value);
	int SDL_WriteBE16(SDL_RWops* dst, Uint16 value);
	int SDL_WriteLE32(SDL_RWops* dst, Uint32 value);
	int SDL_WriteBE32(SDL_RWops* dst, Uint32 value);
	int SDL_WriteLE64(SDL_RWops* dst, Uint64 value);
	int SDL_WriteBE64(SDL_RWops* dst, Uint64 value);
}
